local BaikActionbar = LibStub("AceAddon-3.0"):GetAddon("BaikActionbar")
local MainBar = BaikActionbar:NewModule("MainBar", "AceHook-3.0")
local UiHelper = BaikActionbar.UiHelper
local HideFrames = BaikActionbar.HideFrames
local Width = BaikActionbar.Width

-- Constants
local ExtraBars = {
    MultiBarBottomLeft,
    MultiBarBottomRight
}

-- Constants
local Frames = {
    ArtifactWatchBar,
    HonorWatchBar,
    ReputationWatchBar
}

local function SetupFrames()
    -- Setup MainBar Backdrop
    UiHelper:SetPointFinal(MainMenuBarTexture0, "BOTTOM", MainMenuBarArtFrame, "BOTTOM", -Width.quarter, 0)
    UiHelper:SetPointFinal(MainMenuBarTexture1, "BOTTOM", MainMenuBarArtFrame, "BOTTOM", Width.quarter, 0)

    -- Setup Grphons
    local xOff = Width.half + MainMenuBarLeftEndCap:GetWidth() / 4
    UiHelper:SetPointFinal(MainMenuBarLeftEndCap, "BOTTOM", MainMenuBarArtFrame, "BOTTOM", -xOff, 0)
    UiHelper:SetPointFinal(MainMenuBarRightEndCap, "BOTTOM", MainMenuBarArtFrame, "BOTTOM", xOff, 0)

    -- Setup MainBar Buttons
    xOff = ActionButton1:GetWidth() / 2 - Width.half + 8
    UiHelper:SetPointFinal(ActionButton1, "BOTTOM", MainMenuBarArtFrame, "BOTTOM", xOff, 2)

    -- Hide MainBar Frames
    UiHelper:HideAllFinal(HideFrames)

    -- Setup general watchbar frames
    for _, frame in pairs(Frames) do
        frame:SetWidth(Width.full)
        frame.StatusBar:SetWidth(Width.full)
        frame.StatusBar.XPBarTexture0:SetWidth(Width.quarter)
        frame.StatusBar.XPBarTexture1:SetWidth(Width.quarter)
        frame.StatusBar.XPBarTexture2:SetWidth(Width.quarter)
        frame.StatusBar.XPBarTexture3:SetWidth(Width.quarter)
        frame.StatusBar.WatchBarTexture0:SetWidth(Width.quarter)
        frame.StatusBar.WatchBarTexture1:SetWidth(Width.quarter)
        frame.StatusBar.WatchBarTexture2:SetWidth(Width.quarter)
        frame.StatusBar.WatchBarTexture3:SetWidth(Width.quarter)
    end

    -- Setup XpBar
    MainMenuExpBar_SetWidth(Width.full)
end

local function SetupHook()
    local function Update()
        BaikActionbar:Update()
    end

    -- Set HookScripts for updating ui
    for _, frame in pairs(ExtraBars) do
        frame:HookScript("OnHide", Update)
        frame:HookScript("OnShow", Update)
    end
    MainMenuBarVehicleLeaveButton:HookScript("OnShow", Update)
    --PetActionBarFrame:HookScript("OnShow", Update)
end

local function LoadConfig(value)
    if value then
        UiHelper:Hide(MainMenuBarLeftEndCap)
        UiHelper:Hide(MainMenuBarRightEndCap)
    else
        MainMenuBarLeftEndCap:SetParent(MainMenuBar)
        MainMenuBarRightEndCap:SetParent(MainMenuBar)
    end
end


local mainGroup = {
    name = "MainBar Options",
    type = "group",
    args = {
        gryphon = {
            name = " Hide Gryphons",
            type = "toggle",
            desc = "Toggles Gryphon visibility",
            get = function(info)
                return BaikActionbar.db.profile.hideGryphons
            end,
            set = function(info, value)
                BaikActionbar.db.profile.hideGryphons = value

                LoadConfig(value)
            end
        },
    }
}

-- MainBar API
function MainBar:OnUpdate()
    local leftShown = MultiBarBottomLeft:IsShown()
    local rightShown = MultiBarBottomRight:IsShown()
    local parentFrame = leftShown and MultiBarBottomLeft or MainMenuBar

    UiHelper:SetPoint(MultiBarBottomRight, "BOTTOM", parentFrame, "TOP", 0, 0)

    if rightShown or leftShown then
        parentFrame = rightShown and MultiBarBottomRight or MultiBarBottomLeft
        UiHelper:SetPoint(MainMenuBarVehicleLeaveButton, "BOTTOMRIGHT", parentFrame, "TOPRIGHT", 0, 0)
        UiHelper:SetPoint(PetActionBarFrame, "BOTTOMRIGHT", parentFrame, "TOPRIGHT", 0, 0)
    else
        UiHelper:SetPoint(MainMenuBarVehicleLeaveButton, "BOTTOMRIGHT", MultiBarBottomLeft, "BOTTOMRIGHT", 0, 0)
        UiHelper:SetPoint(PetActionBarFrame, "BOTTOMRIGHT", MultiBarBottomLeft, "BOTTOMRIGHT", 0, 0)
    end
end

-- Ace3 Callbacks
function MainBar:OnInitialize()
    BaikActionbar.options.args["mainGroup"] = mainGroup
    BaikActionbar:DPrint("MainBar OnInitialize")
end

function MainBar:OnEnable()
    SetupFrames()
    SetupHook()
    LoadConfig(BaikActionbar.db.profile.hideGryphons)
    BaikActionbar:DPrint("MainBar OnEnable")
end

function MainBar:OnDisable()
    BaikActionbar:DPrint("MainBar OnDisable")
end
