local BaikActionbar = LibStub("AceAddon-3.0"):GetAddon("BaikActionbar")
local StanceBar = BaikActionbar:NewModule("StanceBar")

-- Imports
local UiHelper = BaikActionbar.UiHelper
local Width = BaikActionbar.Width
local State = BaikActionbar.State
local StateOpts = BaikActionbar.StateOpts
local StateMap = BaikActionbar.StateMap

-- Constants
local Frames = {
    StanceButton1,
    StanceButton2,
    StanceButton3,
    StanceButton4,
    StanceButton5,
    StanceButton6
}

local function SetHide()
    UiHelper:Hide(StanceBarFrame)
end

local function UnsetHide()
    StanceBarFrame:SetParent(MainMenuBar)
end

local function SetHover()
    local function SetInivisble()
        StanceBarFrame:SetAlpha(0.0)
    end

    local function SetVisible()
        StanceBarFrame:SetAlpha(1.0)
    end

    for _, frame in pairs(Frames) do
        frame:SetScript("OnEnter", SetVisible)
        frame:SetScript("OnLeave", SetInivisble)
    end
    StanceBarFrame:SetAlpha(0.0)
end

local function UnsetHover()
    StanceBarFrame:SetAlpha(1.0)

    for _, frame in pairs(Frames) do
        frame:SetScript("OnEnter", nil)
        frame:SetScript("OnLeave", nil)
    end
end

local SetState = {
    [State.hide] = SetHide,
    [State.hover] = SetHover,
    [State.show] = function() end
}

local UnsetState = {
    [State.hide] = UnsetHide,
    [State.hover] = UnsetHover,
    [State.show] = function() end
}

local function LoadConfig(value)
    local curState = StateMap[value]

    if curState then
        SetState[curState]()
    end
end


local stanceGroup = {
    name = "Stance Options",
    type = "group",
    args = {
        bags = {
            name = "Visibility",
            type = "select",
            desc = "Adjust the stance bar visibility",
            values = StateOpts,
            get = function(info)
                return BaikActionbar.db.profile.stance
            end,
            set = function(info, key)
                local curState = StateMap[BaikActionbar.db.profile.stance]
                local newState = StateMap[key]

                -- Update if state changed
                if newState ~= curState then
                    -- Remove last state configurations
                    if curState then
                        UnsetState[curState]()
                    end

                    -- Set new configurations and update key
                    LoadConfig(key)
                    BaikActionbar.db.profile.stance = key
                end
            end
        }
    }
}

-- Ace3 Callbacks
function StanceBar:OnUpdate()
    local leftShown = MultiBarBottomLeft:IsShown()
    local rightShown = MultiBarBottomRight:IsShown()

    if leftShown or rightShown then
        -- Right bar always on top
        local parentFrame = rightShown and MultiBarBottomRight or MultiBarBottomLeft
        UiHelper:SetPoint(StanceBarFrame, "BOTTOMLEFT", parentFrame, "TOPLEFT", 0, 0)
    else
        UiHelper:SetPoint(StanceBarFrame, "BOTTOM", MainMenuBar, "TOP", 16 - Width.half, 0)
    end
end

-- Ace3 Callbacks
function StanceBar:OnInitialize()
    BaikActionbar.options.args["stanceGroup"] = stanceGroup
    BaikActionbar:DPrint("StanceBar OnInitialize")
end

function StanceBar:OnEnable()
    LoadConfig(BaikActionbar.db.profile.stance)
    BaikActionbar:DPrint("StanceBar OnEnable")
end

function StanceBar:OnDisable()
    BaikActionbar:DPrint("StanceBar OnDisable")
end
